<?php

namespace App\Controller\Admin;

use App\Entity\Agence;
use App\Entity\Company;
use App\Entity\Ville;
use Doctrine\ORM\Query\Expr\Join;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\CollectionField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;

class AgenceCrudController extends AbstractCrudController
{
    public static function getEntityFqcn(): string
    {
        return Agence::class;
    }

    public function configureFields(string $pageName): iterable
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        if ($this->isGranted('ROLE_SUPER_ADMIN')){
            $disabled=false;
        }else{
            $disabled=true;
        }
        return [
            AssociationField::new('company', 'Entreprise')
                ->setRequired(true)
                ->setDisabled($disabled),
            AssociationField::new('companyVille', 'Ville')->setQueryBuilder(fn (QueryBuilder $queryBuilder) =>
                $queryBuilder->where('entity.company ='.$companyId)),
            TextField::new('name','Agence'),
        ];
    }

    public function createEntity(string $entityFqcn)
    {
        $user=$this->getUser();
        $company = $user->getCompany();
        $agence=new Agence();
        $agence->setCompany($company);;
        return $agence;
    }
    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();

        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);
        if (!$this->isGranted('ROLE_SUPER_ADMIN')){
            $queryBuilder->where('entity.company = (:company_0)')
                        ->orderBy('entity.companyVille','ASC')
                        ->addOrderBy('entity.name','ASC')
                        ->setParameter('company_0', $companyId);                                                                                                                                                                   
            
        }
        return $queryBuilder;
    }
    
}
