<?php

namespace App\Controller\Admin;

use App\Entity\Bus;
use App\Repository\BusRepository;
use App\Repository\EmployeeRepository;
use Doctrine\ORM\QueryBuilder;
use Doctrine\ORM\Query\AST\WhereClause;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Config\Filters;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\ArrayField;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\DateTimeField;
use EasyCorp\Bundle\EasyAdminBundle\Field\IdField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ImageField;
use EasyCorp\Bundle\EasyAdminBundle\Field\IntegerField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use EasyCorp\Bundle\EasyAdminBundle\Filter\BooleanFilter;
use EasyCorp\Bundle\EasyAdminBundle\Orm\EntityRepository;

class BusCrudController extends AbstractCrudController
{
    public function __construct(private BusRepository $busRepository){}
    public static function getEntityFqcn(): string
    {
        return Bus::class;
    }

    public function configureCrud(Crud $crud): Crud
    {
        return $crud
            ->setPageTitle('index', 'Liste des bus')
            ->setEntityLabelInSingular('ce bus')
            ->setEntityLabelInPlural('Bus')
            ->setDateTimeFormat('dd/MM/yyyy à HH:mm')
            ;
     }
    
     public function createEntity(string $entityFqcn){
        $user=$this->getUser();
        $company = $user->getCompany();
        
        $bus=new Bus();
        $bus->setCompany($company);
        $bus->setEmployee($user);
        $bus->setCreatedAt(new \DateTime());
        $bus->setUpdatedAt(new \DateTime());
        return $bus;
    }

    public function configureFields(string $pageName): iterable
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        $name= $companyId;
        return [
            AssociationField::new('company','Entreprise')->setDisabled(true),
            ImageField::new('image')
                ->setBasePath('images')
                ->setUploadDir('public/assets/img')
                ->setUploadedFileNamePattern("$name.[extension]")
                ->setRequired(false),
            TextField::new('busPlateNumber','Immatriculation'),
            IntegerField::new('capacity','Capacité'),
            AssociationField::new('employee','Employé')->setQueryBuilder(fn (QueryBuilder $queryBuilder) =>
                $queryBuilder->where('entity.company ='.$companyId)),
            AssociationField::new('conforts','Conforts')
                ->setFormTypeOption('choice_label', 'name')
                ->setFormTypeOption('by_reference', false)
            //DateTimeField::new('createdAt')->setFormat('dd/MM/yyyy à HH:mm'),
            //DateTimeField::new('updatedAt')->setFormat('dd/MM/yyyy à HH:mm'),
        ];
    }
    // public function configureFilters(Filters $filters): Filters
    // {
    //     return $filters
    //         ->add('employee')
    //         // most of the times there is no need to define the
    //         // filter type because EasyAdmin can guess it automatically
    //         //->add(BooleanFilter::new('published'))
    //     ;
    // }

    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();

        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);
        if (!$this->isGranted('ROLE_SUPER_ADMIN')){
            $queryBuilder->where('entity.company = (:company_0)')
                        ->setParameter('company_0', $companyId);
        }
        return $queryBuilder;
    }
    
}
