<?php

namespace App\Controller\Admin;

use App\Entity\Company;
use App\Entity\TypeTarif;
use App\Entity\Ville;
use App\Form\TarifType;
use App\Repository\EmployeeRepository;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\CollectionField;
use EasyCorp\Bundle\EasyAdminBundle\Field\IdField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ImageField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;

class CompanyCrudController extends AbstractCrudController
{
    private int $companyId;
    public function __construct(private EmployeeRepository $employeeRepository)
    {
    }
    public static function getEntityFqcn(): string
    {
        return Company::class;
    }

    
    public function configureFields(string $pageName): iterable
    {
        $user=$this->getUser();
        $this->companyId = $user->getCompany()->getId();
        return [
             TextField::new('name','Raison sociale'),
             ImageField::new('image','Image')
                 ->setBasePath('assets/img/company')
                 ->setUploadDir('public/assets/img/company')
                 ->setUploadedFileNamePattern("$this->companyId.[extension]")
                 ->setRequired(false),
             TextField::new('address','Adresse'),
             TextField::new('email','Email'),
             TextField::new('telephone','Téléphone'),
         ];
    }
    
    public function configureCrud(Crud $crud): Crud
    {
        return $crud
            ->setPageTitle('index', 'Liste des compagnies')
            ->setEntityLabelInSingular('Votre compagnie')
            ->setEntityLabelInPlural('compagnies')
            ->setDateTimeFormat('dd/MM/yyyy à HH:mm')
            ;
     }

    public function configureActions(Actions $actions): Actions
    {
         if ($this->isGranted('ROLE_SUPER_ADMIN')){
             return $actions;
         }elseif ($this->isGranted('ROLE_ADMIN'))
         {
             return $actions
             ->disable(Action::NEW, Action::DELETE);
         }else{
            return $actions
            ->disable(Action::NEW, Action::EDIT, Action::DELETE);
        }
    }

    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {  
        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);
        if (!$this->isGranted('ROLE_SUPER_ADMIN')){
            $queryBuilder->where('entity.id = (:company_0)')
                        ->setParameter('company_0', $this->companyId);
        }
        return $queryBuilder;
    }
}
