<?php

namespace App\Controller\Admin;

use App\Entity\CompanyVille;

use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\BooleanField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\EmailField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;


class CompanyVilleCrudController extends AbstractCrudController
{
    public static function getEntityFqcn(): string
    {
        return CompanyVille::class;
    }

    
    public function configureFields(string $pageName): iterable
    {
        if ($this->isGranted('ROLE_SUPER_ADMIN')){
            return [
                AssociationField::new('company'),
                AssociationField::new('ville'),
            ];
        }else{
            return [
                AssociationField::new('company')->setDisabled(true),
                AssociationField::new('ville'),
            ];
        }
    }

    public function createEntity(string $entityFqcn){
        $user=$this->getUser();
        $company = $user->getCompany();
        $companyVille=new CompanyVille();
        $companyVille->setCompany($company);
        //$employee->setEmployee($employee);
        // $employee->setCreatedAt(new \DateTime());
        // $employee->setUpdatedAt(new \DateTime());
        return $companyVille;
    }
    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    { 
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId(); 
        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);
        if (!$this->isGranted('ROLE_SUPER_ADMIN')){
            $queryBuilder->where('entity.company = (:company_0)')
                        ->setParameter('company_0', $companyId);
        }
        return $queryBuilder;
    }

    
}
