<?php

namespace App\Controller\Admin;

use App\Entity\Agence;
use App\Entity\Booking;
use App\Entity\Bus;
use App\Entity\Company;
use App\Entity\CompanyVille;
use App\Entity\Confort;
use App\Entity\Employee;
use App\Entity\Passager;
use App\Entity\Payment;
use App\Entity\Schedule;
use App\Entity\Tarif;
use App\Entity\TypeTarif;
use App\Entity\User;
use App\Entity\Ville;
use App\Form\EmployeeType;
use App\Repository\BookingRepository;
use App\Repository\CompanyRepository;
use App\Repository\EmployeeRepository;
use App\Repository\PassagerRepository;
use App\Repository\PaymentRepository;
use App\Security\EmailVerifier;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Doctrine\ORM\EntityManagerInterface;
use EasyCorp\Bundle\EasyAdminBundle\Config\Dashboard;
use EasyCorp\Bundle\EasyAdminBundle\Config\MenuItem;
use EasyCorp\Bundle\EasyAdminBundle\Config\UserMenu;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractDashboardController;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\IsGranted;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Http\Authentication\AuthenticationUtils;
use Symfony\UX\Chartjs\Builder\ChartBuilderInterface;
use Symfony\UX\Chartjs\Model\Chart;

// #[Route('/admin')]
class DashboardController extends AbstractDashboardController
{

    public function __construct(
        private EmailVerifier $emailVerifier,
        private CompanyRepository $companyRepository,
        private BookingRepository $bookingRepository,
        private ChartBuilderInterface $chartBuilder,
        private EmployeeRepository $employeeRepository,
    )
    {

    }
    private function rgb($val){
        if($val < 0 || $val > 100)
          return; // Error!
      
        $r = (100 - $val)*255/50;
        if((100 - $val) > 50)
          $r = 255;
        $rr = dechex($r);
        if($r < 16)
          $rr = '0' . $rr;
      
        $g = ($val)*255/50;
        if((100-$val) < 50)
          $g = 255;
        $gg = dechex($g);
        if($g < 16)
          $gg = '0'.$gg;
      
        $bb = '00';
      
        return $rr . $gg . $bb;
      }

    #[Route('/admin',name: 'app_admin')]
    public function index(): Response
    {

        $user=$this->getUser();
        if (!$this->isGranted('ROLE_EDITOR')){
            return $this->redirect('/admin/logout');
        }
        if ($user == null){
            //$request->getSession()->set('_referer', '/admin');
            return $this->redirectToRoute('app_admin_login');
        }

        $companyId = $user->getCompany()->getId();
        if (isset($_POST['fromDate']) && isset($_POST['toDate'])){
            $fromDate = $_POST['fromDate'];
            $toDate = $_POST['toDate'];
        }else{
            $fromDate = strtotime(date("Y-m-d", strtotime(date("Y-m-d"))) . " -1 month"); 
            $toDate = new \DateTime;
        }

        isset($_POST['companies']) ? $postCompanies = $_POST['companies'] :  $postCompanies = ['id'=> $companyId];
        //$nCompany = count($postCompanies);
        $companies=[];
        foreach ($postCompanies as $postCompany){
             $companies[] = $this->companyRepository->findOneBy(['id'=> $postCompany]);   
        }

        $bookings = $this->bookingRepository->findByDate($fromDate, $toDate, $companies);
        $data =[];
        $j=0;
        foreach ($bookings as $booking){
            $d = $booking->getDateOfBooking()->format('Y/m/d');
            $mnt = $booking->getTotalAmount();
            $companyId = $booking->getCompany()->getId();
            $data[] = ['date'=>$d, 'company'=>$companyId, 'montant'=>$mnt];
        }
        $dates = array_column($data, 'date');
        $uniqDates = array_unique($dates);
        //dd($uniqDates);
        $montantsCompanies=[];
        foreach ($postCompanies as $postCompany){
            $somme=[];
            foreach($uniqDates as $uniqDate){
                $som=0;
                $datesKeys = array_keys($dates, $uniqDate);
                foreach($datesKeys as $key){
                    if ($data[$key]['company']==$postCompany){
                        $som += $data[$key]['montant'];
                    }
                }
                $somme[]=$som;
            }
            $montantsCompanies[]=$somme;
        }

        if ($this->isGranted('ROLE_SUPER_ADMIN')){
            $companiesAll = $this->companyRepository->findAll();
        }else{
            $companiesAll = $this->companyRepository->findBy(['id'=>$companyId]);
        }


        //$c=123/100;

        $data=[];        
        $i=0;
        foreach ($postCompanies as $postCompany){
            $r = $i*10; 
            $g = $i*5; 
            $b = $i*10; 
            $color = dechex($r) . dechex($g) . dechex($b);

            $color = $this->rgb((($i+4)*20));
            $data[$i]=[
                'label' => $this->companyRepository->findOneBy(['id'=> $postCompany])->getName(),
                'backgroundColor' => '#'.$color,
                'borderColor' => '#'.$color,
                'data' => $montantsCompanies[$i],
            ];
            $i++;
        }


        $dates=[];
        foreach($uniqDates as $uniqDate){
            $dates[]=$uniqDate;
        }

        $chart = $this->chartBuilder->createChart(Chart::TYPE_LINE);
        $chart->setData([
            'labels' => $dates,
            'datasets' => $data,
        ]);

        $chart->setOptions([
            'scales' => [
                'y' => [
                    'suggestedMin' => 0,
                    'suggestedMax' => 100,
                ],
            ],
        ]);

        return $this->render('admin/dashboard.html.twig', [
            'controller_name'=> 'DashboardController',
            'chart' => $chart,
            'fromDate' => $fromDate,
            'toDate' => $toDate,
            'companies' => $companiesAll,
            'postcompanies' => $postCompanies,
        ]);
    }

    public function configureDashboard(): Dashboard
    {
        return Dashboard::new()
            ->setTitle("Kerene's Travel");
    }

    public function configureMenuItems(): iterable
    {
        yield MenuItem::linkToDashboard('Dashboard', 'fa fa-home');
        yield MenuItem::linkToCrud('Planning voyages', 'fas fa-calendar-days', Schedule::class);
        if ($this->isGranted('ROLE_SUPER_ADMIN')){
            yield MenuItem::subMenu('Compagnies', 'fa fa-industry')->setSubItems([
                MenuItem::linkToCrud('Compagnies', 'fas fa-industry', Company::class),
                MenuItem::linkToCrud('Compagnie Villes', 'fas fa-map-location', CompanyVille::class),
                MenuItem::linkToCrud('Ville', 'fas fa-map-location', Ville::class),
                MenuItem::linkToCrud('Agences', 'fas fa-map-location', Agence::class),
                MenuItem::linkToCrud('Bus', 'fas fa-bus', Bus::class)
            ]);
        }else{
            yield MenuItem::subMenu('Compagnies', 'fa fa-industry')->setSubItems([
                MenuItem::linkToCrud('Compagnies', 'fas fa-industry', Company::class),
                MenuItem::linkToCrud('Villes', 'fas fa-map-location', CompanyVille::class),
                MenuItem::linkToCrud('Agences', 'fas fa-map-location', Agence::class),
                MenuItem::linkToCrud('Bus', 'fas fa-bus', Bus::class)
            ]);    
        }
        yield MenuItem::subMenu('Utilisateurs','fas fa-user')->setSubItems([
            MenuItem::linkToCrud('Clients', 'fas fa-user', User::class)->setPermission('ROLE_SUPER_ADMIN'),
            MenuItem::linkToCrud('Employés', 'fas fa-user', Employee::class)
        ]);
        yield MenuItem::subMenu('Tarifs','fas fa-list')->setSubItems([
            MenuItem::linkToCrud('Grille tarifaire', 'fas fa-dollar', Tarif::class),
            MenuItem::linkToCrud('Type de tarif', 'fas fa-dollar', TypeTarif::class),
        ]);

        yield MenuItem::subMenu('Réservations','fas fa-book')->setSubItems([
            MenuItem::linkToCrud('Réservations', 'fas fa-book', Booking::class),
            MenuItem::linkToCrud('Passagers', 'fas fa-user',Passager::class),
            MenuItem::linkToCrud('Paiements', 'fas fa-book', Payment::class)->setPermission('ROLE_ADMIN')
        ]);
        yield MenuItem::linkToCrud('Conforts', 'fas fa-bus', Confort::class)->setPermission('ROLE_SUPER_ADMIN');
        yield MenuItem::section();
        yield MenuItem::linkToRoute('Manifeste', 'fa fa-user', 'app_manifeste');
        yield MenuItem::linkToRoute('Rapport', 'fa fa-user', 'app_report_payment');
        yield MenuItem::linkToRoute('Embarquement', 'fa fa-bus', 'app_embarq');
    }


    public function configureUserMenu(UserInterface $employee): UserMenu
    {
        // Usually it's better to call the parent method because that gives you a
        // user menu with some menu items already created ("sign out", "exit impersonation", etc.)
        // if you prefer to create the user menu from scratch, use: return UserMenu::new()->...
        return parent::configureUserMenu($employee)
            // use the given $user object to get the user name
            ->setName($employee->getLastname().' '. $employee->getFirstname())
            //->setName($employee->getUserIdentifier())
            // use this method if you don't want to display the name of the user
           // ->displayUserName(false)

            // you can return an URL with the avatar image
           // ->setAvatarUrl('https://...')
           // ->setAvatarUrl($user->getProfileImageUrl())
            // use this method if you don't want to display the user image
            //->displayUserAvatar(false)
            // you can also pass an email address to use gravatar's service
           // ->setGravatarEmail($user->getMainEmailAddress())

            // you can use any type of menu item, except submenus
            ->addMenuItems([
                MenuItem::linkToRoute('Profil employé', 'fa fa-user', 'app_employee_profile'),
                //MenuItem::linkToRoute('Settings', 'fa fa-user-cog', 'admin_settings'),
                MenuItem::section(),
                //MenuItem::linkToLogout('Logout', 'fa fa-sign-out'),
            ]);
    }
 
    #[Route(path: '/admin/login', name: 'app_admin_login')]
    //#[IsGranted("PUBLIC_ACCESS")]
    public function login(Request $request, AuthenticationUtils $authenticationUtils): Response
    {
        //$targetPath = $request->getSession()->get('_security.main.target_path');  
        $targetPath = $request->getSession()->get('_referer');
        // get the login error if there is one
        $error = $authenticationUtils->getLastAuthenticationError();
        // dd($error);
        // last username entered by the user
        $lastUsername = $authenticationUtils->getLastUsername();
        //dd($lastUsername, $error);
        return $this->render('security/login.html.twig', [
            'last_username' => $lastUsername,
            'error' => $error,
            // OPTIONAL parameters to customize the login form:

            // the translation_domain to use (define this option only if you are
            // rendering the login template in a regular Symfony controller; when
            // rendering it from an EasyAdmin Dashboard this is automatically set to
            // the same domain as the rest of the Dashboard)
            //'translation_domain' => 'admin',

            // by default EasyAdmin displays a black square as its default favicon;
            // use this method to display a custom favicon: the given path is passed
            // "as is" to the Twig asset() function:
            // <link rel="shortcut icon" href="{{ asset('...') }}">
            //'favicon_path' => '/favicon-admin.svg',

            // the title visible above the login form (define this option only if you are
            // rendering the login template in a regular Symfony controller; when rendering
            // it from an EasyAdmin Dashboard this is automatically set as the Dashboard title)
            'page_title' => 'Kerene\'s Digital',

            // the string used to generate the CSRF token. If you don't define
            // this parameter, the login form won't include a CSRF token
            'csrf_token_intention' => 'authenticate',

            // the URL users are redirected to after the login (default: '/admin')
            'target_path' => $targetPath,

            // the label displayed for the username form field (the |trans filter is applied to it)
            //'username_label' => 'Your username',

            // the label displayed for the password form field (the |trans filter is applied to it)
            //'password_label' => 'Your password',

            // the label displayed for the Sign In form button (the |trans filter is applied to it)
            //'sign_in_label' => 'Log in',

            // the 'name' HTML attribute of the <input> used for the username field (default: '_username')
            //'username_parameter' => 'my_custom_username_field',

            // the 'name' HTML attribute of the <input> used for the password field (default: '_password')
            //'password_parameter' => 'my_custom_password_field',

            // whether to enable or not the "forgot password?" link (default: false)
            'forgot_password_enabled' => true,

            // the path (i.e. a relative or absolute URL) to visit when clicking the "forgot password?" link (default: '#')
            'forgot_password_path' => $this->generateUrl('app_forgot_password_request'),

            // the label displayed for the "forgot password?" link (the |trans filter is applied to it)
            //'forgot_password_label' => 'Forgot your password?',

            // whether to enable or not the "remember me" checkbox (default: false)
            'remember_me_enabled' => true,
            'register_path' => $this->generateUrl('app_register'),

            // remember me name form field (default: '_remember_me')
            'remember_me_parameter' => 'custom_remember_me_param',

            // whether to check by default the "remember me" checkbox (default: false)
            'remember_me_checked' => true,

            // the label displayed for the remember me checkbox (the |trans filter is applied to it)
            //'remember_me_label' => 'Remember me',
            ]);
    }

    #[Route(path: '/admin/logout', name: 'app_admin_logout')]
    public function logout(): void
    {
        throw new \LogicException('This method can be blank - it will be intercepted by the logout key on your firewall.');
    }



    #[Route('/admin/profile', name: 'app_employee_profile', methods: ['GET', 'POST'])]
    public function new(Request $request, EntityManagerInterface $entityManager, EmployeeRepository $er): Response
    {
        $employee= $er->findOneBy(['id'=>$this->getUser()->getId()]);
        $form = $this->createForm(EmployeeType::class, $employee);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->persist($employee);
            $entityManager->flush();
        }
        return $this->renderForm('admin/profile.html.twig', [
            'form' => $form,
        ]);
    }

    #[IsGranted("ROLE_EDITOR")]
    #[Route('/admin/embarq', name: 'app_embarq', methods: ['GET', 'POST'])]
    public function embarq(
        Request $request,
        //Booking $booking,
        PaymentRepository $paymentRepository,
        BookingRepository $bookingRepository,
        PassagerRepository $passagerRepository,
        EntityManagerInterface $entityManager) : Response
    {
        $user=$this->getUser();
        $message='';
        if ($user == null){
            return $this->redirectToRoute('app_admin_login');
        }
        if (!$this->isGranted('ROLE_EDITOR')){
            return $this->redirect('/admin/logout');
        }

        $form1 = $this->createFormBuilder()
             ->add('numdossier', TextType::class, [
                'label' => false,
                'disabled' => false,
            ])
            ->add('send1', SubmitType::class,[
                'label' => 'Ok',
                'attr' =>[
                    'class'=>'btn btn-primary',
                ]
            ])
            ->getForm();

        $numdossier='';
        $form1->handleRequest($request);
        if ($form1->isSubmitted() && $form1->isValid()) {
            $numdossier = $request->request->all()['form']['numdossier'];
        }

        $user = $this->getUser();
        $companyId = $user->getCompany()->getId();
        $booking = $bookingRepository->findOneBy(['numdossier'=> $numdossier, 'company'=> $companyId]);
        if ($booking == null){
            $payment = $paymentRepository->findOneBy(['paymentMethod'=>$numdossier, 'company'=> $companyId]);
            if ($payment == null){
                $id=0;
            }else{
                $booking = $bookingRepository->findOneBy(['id'=> $payment->getBooking()->getId(), 'company'=> $companyId]);
                $id = $booking->getId();    
            }
        }else{
            $id = $booking->getId();
        }
        $disabled=false;
        $form2 = $this->createFormBuilder()
             ->add('passagers', EntityType::class, [
                'class' => Passager::class,
                'label' => 'Passager à embarquer',
                'disabled' => false,
                'query_builder' => function (PassagerRepository $pr)use($id) {
                    return $pr->createQueryBuilder('p')
                        ->where('p.booking = :bookingId')
                        ->andwhere('p.employeeEmbarq is null')
                        ->setParameter('bookingId', $id);
                },
                'expanded' => true,
                'multiple' => true,
            ])
            ->add('send2', SubmitType::class,[
                'label' => 'Embarquer',
                'disabled' => $disabled,
                'attr' =>[
                    'class'=>'btn btn-warning',
                    'placeholder' => 'Embarquer',
                ]
            ])
            ->getForm();

        $form2->handleRequest($request);        
        if ($form2->isSubmitted()) {
            if (isset($request->request->all()['form']['passagers'])){
                $pids = $request->request->all()['form']['passagers'];
                foreach ($pids as $pid){
                    $passager = $passagerRepository->findOneBy(['id'=> $pid]);
                    $passager->setEmbarqDate(new \DateTime);
                    $passager->setEmployeeEmbarq($user);
                    $entityManager->flush();
                    $this->addFlash('success', 'Embarquement réussi de '. $passager->getLastname().' '. $passager->getFirstname());
                }
                return $this->redirectToRoute('app_embarq');
            }else{
                $message = "Pas de passager à embarquer";
            };
        }
        if ($booking==null){
            $message = "Aucune réservation ne correspond à ce dossier";
        }
        return $this->renderForm('admin/embarq.html.twig', [
            'form1' => $form1,
            'form2' => $form2,
            'user' => $user,
            'message'=>$message,
            'booking' => $booking,
        ]);        
    }    
}
