<?php

namespace App\Controller\Admin;

use App\Entity\Tarif;
use App\Entity\Ville;
use App\Repository\CompanyRepository;
use App\Repository\CompanyVilleRepository;
use App\Repository\EmployeeRepository;
use App\Repository\StopoverRepository;
use App\Repository\TarifRepository;
use App\Repository\TypeTarifRepository;
use App\Repository\VilleRepository;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Config\KeyValueStore;
use EasyCorp\Bundle\EasyAdminBundle\Context\AdminContext;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\MoneyField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;

class TarifCrudController extends AbstractCrudController
{
    public function __construct(
            private EmployeeRepository $employeeRepository,
            private TarifRepository $tarifRepository,
            private TypeTarifRepository $typeTarifRepository,
            private CompanyVilleRepository $companyVilleRepository,
            private villeRepository $villeRepository,
            private CompanyRepository $companyRepository,
        ){}
    public static function getEntityFqcn(): string
    {
        return Tarif::class;
    }
    public function configureCrud(Crud $crud): Crud
    {
        return $crud->overrideTemplate('crud/index', 'admin/tarifs.html.twig')
                    ->setPageTitle('index', 'Grille tarifaire')
                    ->setEntityLabelInSingular('ce tarif')
                    ->setEntityLabelInPlural('Tarifs')
                    ->setDateTimeFormat('dd/MM/yyyy à HH:mm')
        ;
}

    public function configureResponseParameters(KeyValueStore $responseParameters): KeyValueStore
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();

        $tarifs = $this->tarifRepository->findBy(['company'=> $companyId]);
        $typeTarifs = $this->typeTarifRepository->findBy(['company'=> $companyId]);

        // $stopovers = $this->villeRepository->findAll();

        // $stopovers = $this->villeRepository->createQueryBuilder('v')
        //     ->innerJoin('v.companies', 'c')
        //     ->where('c.id = :company_id')
        //     ->orderBy('v.name','ASC')
        //     ->setParameter('company_id', $companyId)
        //     ->getQuery()->getResult();


        $stopovers = $this->companyVilleRepository->findBy(['company'=> $companyId]);
        $arrays=[];
        $i=0;
        foreach($stopovers as $stopoverD){
            //$arrays[] = ['escale'=>$stopoverD->getName()];
            $j=0;
            foreach($stopovers as $stopoverA){           
                $k=0;
                foreach($typeTarifs as $typeTarif){
                    $arrays[$i][$j][$k]=[];
                    foreach($tarifs as $tarif){
                        if ($tarif->getDepart()->getId() == $stopoverD->getVille()->getId() && 
                            $tarif->getArrivee()->getId() == $stopoverA->getVille()->getId() &&
                            $tarif->getTypeTarif()->getId() == $typeTarif->getId()){
                            $arrays[$i][$j][$k][] = ['depart' => $stopoverD->getVille()->getId(),
                                        'arrivee' => $stopoverA->getId(),
                                        'typeTarif' => $tarif->getTypeTarif()->getName(),
                                        'tarifName' => $tarif->getName(),
                                        'tarifId' => $tarif->getId(),
                                        'montant' => $tarif->getAmount()
                                        ];
                        }
                    }
                    $k++;
                }
                $j++;    
            }
            $i++;
        }
        $responseParameters->set('tarifs', $this->tarifRepository->findBy(['company'=> $companyId]));
        $responseParameters->set('stopovers', $stopovers);
        $responseParameters->set('typeTarifs', $typeTarifs);
        $responseParameters->set('arrays', $arrays);
        return $responseParameters;
    }
    public function createEntity(string $entityFqcn){
        $departId = ($_GET['d']);
        $arriveeId = ($_GET['a']);
        $typeTarifId = ($_GET['t']);
        $user=$this->getUser();
        $company = $user->getCompany();
        $depart = $this->villeRepository->findOneBy(['id'=> $departId]);
        $arrivee = $this->villeRepository->findOneBy(['id'=> $arriveeId]);
        $typeTarif = $this->typeTarifRepository->findOneBy(['id'=> $typeTarifId]);

        $tarif=new Tarif();
        $tarif->setCompany($company);
        $tarif->setDepart($depart);
        $tarif->setArrivee($arrivee);
        $tarif->setTypeTarif($typeTarif);
        $tarif->setCreatedAt(new \DateTime());
        $tarif->setUpdatedAt(new \DateTime());

        return $tarif;
    }
    
    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        
        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);   
        $queryBuilder->where('entity.company = (:company_0)')
                     ->setParameter('company_0', $companyId);
        return $queryBuilder;
    }
    public function configureFields(string $pageName): iterable
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        return [
            AssociationField::new('depart','Départ')
                ->setRequired(true)
                ->setDisabled(true),
            AssociationField::new('arrivee','Arrivée')
                ->setRequired(true)
                ->setDisabled(true),
            AssociationField::new('typeTarif', 'Type tarif')
                ->setRequired(true)
                ->setQueryBuilder(fn (QueryBuilder $queryBuilder) =>$queryBuilder->where('entity.company ='.$companyId)
            ),
            TextField::new('name','Tarif'),
            MoneyField::new('amount','Montant')->setCurrency("XAF")
                ->setCustomOption('storedAsCents', false)
                ->setNumDecimals(0),
        ];
    }
    public function configureActions(Actions $actions): Actions
    {
        if ($this->isGranted('ROLE_ADMIN')){
            return $actions
                ->add(Crud::PAGE_EDIT, Action::DELETE);
        }else{
            return $actions
                ->disable(Action::NEW, Action::EDIT, Action::DELETE);
        }
    }
}
