<?php

namespace App\Controller\Admin;

use App\Entity\Tarif;
use App\Entity\TypeTarif;
use App\Repository\EmployeeRepository;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\CollectionField;
use EasyCorp\Bundle\EasyAdminBundle\Field\IdField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;

class TypeTarifCrudController extends AbstractCrudController
{
    
    public function __construct(private EmployeeRepository $employeeRepository){}
    public static function getEntityFqcn(): string
    {
        return TypeTarif::class;
    }

    public function configureCrud(Crud $crud): Crud
    {
        return $crud
            ->setPageTitle('index', 'Liste des types de tarifs')
            ->setEntityLabelInSingular('ce type de tarif')
            ->setEntityLabelInPlural('tarifs')
            ->setDateTimeFormat('dd/MM/yyyy à HH:mm')
            ;
     }

     public function createEntity(string $entityFqcn){
        $user=$this->getUser();
        $company = $user->getCompany();
        
        $typeTarif=new TypeTarif();
        $typeTarif->setCompany($company);
        //$employee->setEmployee($employee);
        // $typeTarif->setCreatedAt(new \DateTime());
        // $typeTarif->setUpdatedAt(new \DateTime());
        return $typeTarif;
    }

    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        
        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);   
        $queryBuilder->where('entity.company = (:company_0)')
                     ->setParameter('company_0', $companyId);
        return $queryBuilder;
    }
    
    public function configureFields(string $pageName): iterable
    {
        return [
            AssociationField::new('company','Entreprise')->setDisabled(true),
            TextField::new('name','Type'),
        ];
    }
    public function configureActions(Actions $actions): Actions
    {
        if ($this->isGranted('ROLE_ADMIN')){
            return $actions;
        }else{
            return $actions
            ->disable(Action::NEW, Action::EDIT, Action::DELETE);
        }
    }
}
