<?php

namespace App\Controller;

use App\Entity\Agence;
use App\Entity\Booking;
use App\Entity\Passager;
use App\Entity\Payment;
use App\Entity\Schedule;
use App\Form\BookingType;
use App\Repository\AgenceRepository;
use App\Repository\BookingRepository;
use App\Repository\CompanyVilleRepository;
use App\Repository\PassagerRepository;
use App\Repository\PaymentRepository;
use App\Repository\ScheduleRepository;
use App\Repository\TarifRepository;
use App\Repository\UserRepository;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use Knp\Component\Pager\PaginatorInterface;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Output\ConsoleOutput;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Http\Attribute\IsGranted;

#[Route('/booking')]
class BookingController extends AbstractController
{
    
    #[IsGranted('ROLE_USER')]
    #[Route('/', name: 'app_booking_index', methods: ['GET'])]
    public function index(
            BookingRepository $bookingRepository,
            UserRepository $ur,
            PaginatorInterface $paginator,
            Request $request
    ): Response
    {

        $id=$this->getUser()->getUserIdentifier();;
        $user = $ur->findOneBy(['email'=> $id]);
        $data = $bookingRepository->findBy(
            ['user' => $user->getId()], ['dateOfBooking'=>'DESC']
        );
        $bookings = $paginator->paginate(
            $data,
            $request->query->getInt('page', 1), 10
        );

        //--------------------------------------------------------------------------
        return $this->render('booking/index.html.twig', [
            'bookings' => $bookings]);
    }


    #[Route('/convert', name: 'app_booking_convert', methods: ['GET', 'POST'])]
    public function convert(Request $request,
        PaymentRepository $paymentRepository,
        BookingRepository $bookingRepository,
        EntityManagerInterface $entityManager): void
    {
        $bookings = $bookingRepository->findAll();
        foreach ($bookings as $booking){
            $params = $booking->getParams();
            if ($params){
                $uuid=$params['uuid'];
                $payment = $paymentRepository->findOneBy(['paymentReference'=> $uuid]);
                if ($payment){
                    $payment->setBooking($booking);
                    $payment->setCompany($booking->getCompany());
                    $entityManager->persist($payment);
                    $entityManager->flush();
                }
            }
        }
        dd('fin');
    }

    #[Route('/convert2', name: 'app_booking_convert2', methods: ['GET', 'POST'])]
    public function convert2(Request $request,
        UserRepository $userRepository,
        BookingRepository $bookingRepository,
        EntityManagerInterface $entityManager): void
    {
        $bookings = $bookingRepository->findAll();
        $users = $userRepository->findAll();
        $i=0;
        foreach ($bookings as $booking){
            $booking->setUser($users[$i]);
            $entityManager->persist($booking);
            $entityManager->flush();
            $i=$i+1;
            if ($i==47){
                $i=0;
            }
        }
        dd('fin');
    }

    #[Route('/convert3', name: 'app_booking_convert3', methods: ['GET', 'POST'])]
    public function convert3(Request $request,
        UserRepository $userRepository,
        PaymentRepository $paymentRepository,
        EntityManagerInterface $entityManager): void
    {
        $payments = $paymentRepository->findAll();
        $users = $userRepository->findAll();
        $i=0;
        foreach ($payments as $payment){
            $transaction = $payment->getTransaction();
            $p=strpos($transaction,'/',0);
            if ($p>0){
                $s = substr($transaction,$p+1);
                $payment->setTransaction($s);
                $entityManager->persist($payment);
                $entityManager->flush();
            }
        }
        dd('fin');
    }



    #[Route('/search', name: 'app_booking_search', methods: ['GET', 'POST'])]
    public function search(Request $request,
        PassagerRepository $passagerRepository,
        PaymentRepository $paymentRepository,
        BookingRepository $bookingRepository): Response
    {
        $form = $this->createFormBuilder()
            ->add('book', TextType::class,[
                'label' => 'Numéro réservation ou Numéro transaction',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    'placeholder' => 'ex. AB1234 ou 123456789',
                ]
            ])
            ->add('lastname', TextType::class,[
                'label' => 'Nom :',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    'placeholder' => 'ex. LOEMBA',
                ]
            ])
            ->add('send', SubmitType::class,[
                'label' => 'Rechercher',
                'attr' =>[
                    'class'=>'btn btn-primary btn-lg btn-block',
                    'placeholder' => 'Rechercher',
                ]
            ])
            ->getForm();

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $data = $request->request->all();
            $book = $data['form']['book'];
            $lastname = $data['form']['lastname'];
            $payment = $paymentRepository->findOneBy(['transaction'=>$book]);
            if ($payment === null) {
                $booking = $bookingRepository->findOneBy(['numdossier'=>$book]);
            } else {
                $booking = $bookingRepository->findOneBy(['id'=>$payment->getBooking()->getId()]);
            }

            if ($booking){
                $passager = $passagerRepository->findOneBy(['booking'=>$booking->getId(), 'lastname'=>$lastname]);
                if ($passager){
                    $booking = $bookingRepository->findOneBy(['id'=>$passager->getBooking()->getId()]);
                    $passagers = $passagerRepository->findBy(['booking'=>$booking->getId()]);
                    //dd($passagers);
                    $qrcode = '/assets/qr-code/'.$booking->getQrcode();
                    return $this->render('booking/result.html.twig', [
                        'form' => $form,
                        'qrcode' => $qrcode,
                        'passagers' => $passagers,
                        'booking' => $booking,
                    ]);
                }
            }
            return $this->render('booking/result-no.html.twig', ['form' => $form]);
        }

        return $this->render('booking/search.html.twig', [
            'form' => $form,
        ]);
    }


    #[Route('/scan', name: 'app_booking_scan', methods: ['GET'])]
    public function scan(): Response
    {
        return $this->render('booking/scan.html', [
        ]);
    }


    #[Route('/{id}', name: 'app_booking_show', methods: ['GET'])]
    public function show(Booking $booking): Response
    {
        return $this->render('booking/show.html.twig', [
            'booking' => $booking,
        ]);
    }

    #[Route('/{id}/edit', name: 'app_booking_edit', methods: ['GET', 'POST'])]
    public function edit(Request $request, Booking $booking, EntityManagerInterface $entityManager): Response
    {
        $form = $this->createForm(BookingType::class, $booking);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->flush();

            return $this->redirectToRoute('app_booking_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->render('booking/edit.html.twig', [
            'booking' => $booking,
            'form' => $form,
        ]);
    }

    #[Route('/{id}', name: 'app_booking_delete', methods: ['POST'])]
    public function delete(Request $request, Booking $booking, EntityManagerInterface $entityManager): Response
    {
        if ($this->isCsrfTokenValid('delete'.$booking->getId(), $request->request->get('_token'))) {
            $entityManager->remove($booking);
            $entityManager->flush();
        }

        return $this->redirectToRoute('app_booking_index', [], Response::HTTP_SEE_OTHER);
    }
    private function numdossier(){
        $index = '1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $base= strlen($index);
        $num='';
        for ($i=0; $i<6; $i++) { 
            $d=rand(1,$base); 
            $num=$num.$index[$d-1]; 
        }
        return $num;
    }

    //#[IsGranted("ROLE_USER")]
    #[Route('/book/{id}', name: 'app_booking_book', methods: ['GET', 'POST'])]
    public function book(
                Request $request,
                Schedule $schedule,
                TarifRepository $tr,
                PassagerRepository $pr,
                ScheduleRepository $scheduleRepository,
                CompanyVilleRepository $companyVilleRepository,
                PaymentController $paymentCtrl,
                AgenceRepository $agenceRepository,
                EntityManagerInterface $entityManager): Response
    {
        $user=$this->getUser();
        $booking = new Booking();
        $booking->setNumdossier($this->numdossier());

        $numberOfSeats=1;
        $booking->setDateOfBooking(new \DateTime());
        $booking->setSchedule($schedule);
        $booking->setNumberOfSeats($numberOfSeats);
        $booking->setCompany($schedule->getCompany());
        if ($user){
            $booking->setUser($user);
            $phone =$user->getPhone();
        }else{
            $phone='';
        };
        $t_env=$_ENV['T_ENV'];
        if ($t_env=='sandbox'){
            $apiurl = $_ENV['API_URL_DEV'];
        }else{
            $apiurl=$_ENV['API_URL'];
        }
        $companyId = $booking->getCompany()->getId();
        $villeDepartId = $booking->getSchedule()->getDepart()->getId();
        $villeArriveeId = $booking->getSchedule()->getArrivee()->getId();
        $companyVilleDepart = $companyVilleRepository->findOneBy(['company'=>$companyId, 'ville'=>$villeDepartId])->getId();
        $companyVilleArrivee = $companyVilleRepository->findOneBy(['company'=>$companyId, 'ville'=>$villeArriveeId])->getId();
        $form = $this->createFormBuilder()
            ->add('agenceDepart', EntityType::class, [
                'placeholder' => 'Agence de départ',
                'class' => Agence::class,
                'query_builder' => function (EntityRepository $er) use($companyVilleDepart) {
                    return $er->createQueryBuilder('a')
                        ->where('a.companyVille = '.$companyVilleDepart);
                },
                'choice_label'  => 'name',
                'choice_value'  => 'id',
                // 'mapped'=> true,
                'label' => 'Départ de '.$booking->getSchedule()->getDepart(),
                'required' => true
            ])
            ->add('agenceArrivee', EntityType::class, [
                'placeholder' => 'Agence d\'arrivée',
                'class' => Agence::class,
                'query_builder' => function (EntityRepository $er) use($companyVilleArrivee) {
                    return $er->createQueryBuilder('a')
                        ->where('a.companyVille = '.$companyVilleArrivee);
                },
                'choice_label'  => 'name',
                'choice_value'  => 'id',
                // 'mapped'=> true,
                'label' => 'Arrivée à '.$booking->getSchedule()->getArrivee(),
                'required' => true
            ])
            ->add('phone', TextType::class, [
                'label' => false,
                'data' => $phone,
                'attr' => [
                    'maxlength' => 9,
                    'pattern' => '[0]{1}[4-6]{1}[0-9]{7}',
                    'placeholder' => 'n° téléphone MTN',   
                ],
            ])
            ->add('send', SubmitType::class,[
                'label' => 'Paiement via Momo',
                'label_html' => true,
                'attr' =>[
                    'class'=>'btn btn-warning',
                    'placeholder' => 'Paiement',
                ],

            
            ])
            ->getForm();

 
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $data = $request->request->all();
            $phone = '242'.$data['form']['phone'];
            $firstname = $data['firstname'];
            $lastname = $data['lastname'];
            $agenceDepart = $data['form']['agenceDepart'];
            $agenceArrivee = $data['form']['agenceArrivee'];
            $ages = $data['age'];
            $numberOfSeats = count($ages);
            // calcul TotalAmount
            $totalAmount = 0;
            foreach ($ages as $age) {
                $tarif = $tr->findOneBy(['id' => $age]);
                $totalAmount += $tarif->getAmount();
            }

            if ($t_env=='sandbox'){
                $currency=$_ENV['CURRENCY_DEV'];
                $uuid = $paymentCtrl->gen_uuid();
                $oask=$_ENV['OASK_DEV'];
                $code = $paymentCtrl->userCreate($apiurl, $uuid, $oask);
                if ($code==201){
                    $apiuser= $uuid;
                    $apikey = $paymentCtrl->apiUser($apiurl, $uuid, $oask);
                }else{
                    return $this->redirectToRoute('app_home');
                }
            }else{
                $currency=$_ENV['CURRENCY'];
                $uuid = $paymentCtrl->gen_uuid();
                $oask=$_ENV['OASK'];
                $apiuser = $_ENV['API_USER'];
                $apikey = $_ENV['API_KEY'];
            }
            $acs_token = $paymentCtrl->accessToken($apiurl, $apiuser, $apikey, $oask);
            $params = [
                'acs_token'=> $acs_token,
                'uuid'=> $uuid,
            ];
            $booking->setParams($params);
            $booking->setNumberOfSeats($numberOfSeats);
            $booking->setTotalAmount($totalAmount);
            $booking->setAgenceDepart($agenceRepository->findOneBy(['id'=>$agenceDepart]));
            $booking->setAgenceArrivee($agenceRepository->findOneBy(['id'=>$agenceArrivee]));
            $entityManager->persist($booking);
            for ($i=0 ; $i<$numberOfSeats; $i++){
                $passager = new Passager;
                $passager->setFirstname($firstname[$i]);
                $passager->setLastname($lastname[$i]);
                $passager->setAge($ages[$i]);
                $passager->setBooking($booking); 
                $passager->setCompany($schedule->getCompany());
                $entityManager->persist($passager);
            }               

            $code = $paymentCtrl->requestToPay($apiurl, $totalAmount, $currency, $phone, $uuid, $acs_token, $oask, $t_env);
            if ($code == 202){  
                $entityManager->flush();
                //$this->addFlash('payment', 'Merci de valider le paiement sur votre téléphone. Tapez *105#');

               // return $this->redirectToRoute('app_payment_reqstatus', ["id" => $booking->getId()], Response::HTTP_SEE_OTHER);
                $request = $paymentCtrl->requestToPayStatus($apiurl, $uuid, $acs_token, $oask, $t_env);

                $output = new ConsoleOutput();
                $output->setVerbosity(OutputInterface::VERBOSITY_VERBOSE);
                $progressBar = new ProgressBar($output, 50);
                $progressBar->start();
                do
                {
                    $response = $request->send();
                    $code = $response->getStatus();
                    $progressBar->advance();
                    $body = $response->getBody();
                    $bdy = json_decode($body, true); 
                    $status= $bdy['status'];
                    sleep(2);
                }while ($status=='PENDING');
                $progressBar->finish();
    
                if ($status=='SUCCESSFUL') {   
                    $booking->setStatus($bdy);
                    $payment = new Payment();
                    $payment->setPaymentDate(new \DateTime());
                    $payment->setBooking($booking);
                    $payment->setCompany($booking->getCompany());
                    $payment->setAmountPaid($booking->getTotalAmount()); 
                    $payment->setTransaction($bdy['financialTransactionId']);
                    $payment->setPaymentReference($bdy['externalId']);
    
                    $capacity= $booking->getSchedule()->getSeats();
                    $schedule = $scheduleRepository->findOneBy(['id'=> $booking->getSchedule()->getId()]);
                    $schedule->setSeats($capacity - $booking->getNumberOfSeats());
    
                    $passagers =$pr->findBy(['booking'=> $booking->getId()]);
    
                    $qrcode= $paymentCtrl->qrCode($booking->getParams()['uuid'], $booking->getNumdossier());
                    $booking->setQrcode($qrcode);
                    $pdfFacture = $paymentCtrl->createFacturePdf($booking, $passagers);
                    $booking->setFacture($pdfFacture);
                    if ($user){
                        if ($user->getPhone()== '') {
                            $user->setPhone($phone);
                            $entityManager->persist($user);
                        }    
                    }
                    $entityManager->persist($payment);
                    $entityManager->persist($booking);
                    $entityManager->persist($schedule);
                    $entityManager->flush();
        
                    return $this->redirectToRoute('app_payment_status', ["id" => $booking->getNumdossier()], Response::HTTP_SEE_OTHER);
                }else{
                    $this->addFlash('echec', 'Le paiement n\'a pas abouti. Votre compte ne sera pas débité !');
                    return $this->redirectToRoute('app_booking_book', ["id"=>$booking->getSchedule()->getId()], Response::HTTP_SEE_OTHER);
                }
            }else{
                $this->addFlash('echec', 'Echec du paiement. Votre compte ne sera pas débité !');
            }
        }
        return $this->render('booking/book.html.twig', [
            'booking' => $booking,
            'user' => $user,
            'form' => $form,
       ]);
    }
}
