<?php

namespace App\Controller;


use App\Entity\Booking;
use App\Entity\Payment;
use App\Entity\User;
use App\Repository\BookingRepository;
use App\Repository\PassagerRepository;
use App\Repository\PaymentRepository;
use App\Repository\UserRepository;
use App\Security\LoginAuthAuthenticator;
use DateTime;
use Doctrine\ORM\EntityManagerInterface;
use Dompdf\Dompdf;
use Dompdf\Options;
use Endroid\QrCode\Builder\BuilderInterface;
use Endroid\QrCode\Encoding\Encoding;
use Endroid\QrCode\Label\Font\NotoSans;
use HTTP_Request2;
use Knp\Component\Pager\PaginatorInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Http\Attribute\IsGranted;
use Symfony\Component\Security\Http\Authentication\UserAuthenticatorInterface;
use Symfony\Component\Security\Http\Util\TargetPathTrait;


#[Route('/payment')]
class PaymentController extends AbstractController
{
    use TargetPathTrait;

    protected $builder ; 
    public function __construct(BuilderInterface $builder)
    {
        $this->builder = $builder;
    }
    
    public function gen_uuid()
    {
        return sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
    
          // 32 bits for "time_low"
          mt_rand(0, 0xffff), mt_rand(0, 0xffff),
    
          // 16 bits for "time_mid"
          mt_rand(0, 0xffff),
    
          // 16 bits for "time_hi_and_version",
          // four most significant bits holds version number 4
          mt_rand(0, 0x0fff) | 0x4000,
    
          // 16 bits, 8 bits for "clk_seq_hi_res",
          // 8 bits for "clk_seq_low",
          // two most significant bits holds zero and one for variant DCE1.1
          mt_rand(0, 0x3fff) | 0x8000,
    
          // 48 bits for "node"
          mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
        );
    }
    public function userCreate($urlrequ, $X_Reference_Id, $Ocp_Apim_Subscription_Key)
    {
        $request = new Http_Request2($urlrequ . '/v1_0/apiuser');
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'X-Reference-Id' => $X_Reference_Id,
            'Content-Type' => 'application/json',
            'Ocp-Apim-Subscription-Key' => $Ocp_Apim_Subscription_Key,
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(HTTP_Request2::METHOD_POST);

        // Request body
        $request->setBody('{
            "providerCallbackHost": "string"
          }');

        try
        {
            $response = $request->send();
            return $response->getStatus() ;// echo $response->getBody();
        }
        catch (HttpException $ex)
        {
            echo $ex;
        }
        return 0;
    }

    public function apiUser($urlrequ, $X_Reference_Id, $Ocp_Apim_Subscription_Key)
    {
        $request = new Http_Request2($urlrequ . '/v1_0/apiuser/' . $X_Reference_Id . '/apikey');
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            //'X-Reference-Id' => $X_Reference_Id,
            //'Content-Type' => 'application/json',
            'Ocp-Apim-Subscription-Key' => $Ocp_Apim_Subscription_Key,
        );

        $request->setHeader($headers);

        $parameters = array(
            
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(HTTP_Request2::METHOD_POST);

        try
        {
            $response = $request->send();
            return json_decode($response->getBody())->apiKey;
        }
        catch (HttpException $ex)
        {
            echo $ex;
        }
    }
    public function qrCode($uuid, $numdossier)
    {
        $dir =$_SERVER['SERVER_NAME'].'/admin/embarq/'.$numdossier;
        $result = $this->builder
            ->data($dir)
            ->size(220)
            ->margin(10)
            ->encoding(new Encoding('UTF-8'))
            //->errorCorrectionLevel( new ErrorCorrectionLevelHigh)
            ->labelText($uuid)
            ->labelFont(new NotoSans(8))
            ->build();

        $namePng = uniqid(prefix:'', more_entropy: false).'.png';
        $result->saveToFile(dirname(__DIR__, 2).'/public/assets/qr-code/'.$namePng);
        return $namePng;

    }


    #[Route('/status/{id}', name: 'app_payment_status', methods: ['GET', 'POST'])]
    public function paymentStatus(Request $request,
         BookingRepository $bookingRepository,
         UserRepository $userRepository,
         PassagerRepository $passagerRepository,
         UserPasswordHasherInterface $userPasswordHasher,
         MailerInterface $mailer,
         UserAuthenticatorInterface $userAuthenticator,
         LoginAuthAuthenticator $authenticator,
         EntityManagerInterface $entityManager):Response
    {

        $numdossier = $request->get('id');
        $booking=$bookingRepository->findOneBy(['numdossier'=>$numdossier]);
        $form = $this->createFormBuilder()
            ->add('email', EmailType::class,[
                'label' => 'Email',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    'placeholder' => 'nom@domain.com',
                ]
            ])
            ->add('password', PasswordType::class,[
                'label' => 'Mot de passe',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    'placeholder' => 'Mot de passe',
                ]
            ])
            ->add('send', SubmitType::class,[
                'label' => 'Valider',
                'attr' =>[
                    'class'=>'btn btn-warning',
                    'placeholder' => 'Valider',
                ]
            ])
            ->getForm();

        $passagers =$passagerRepository->findBy(['booking'=> $booking->getId()]);
        $user = $this->getUser();
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $data= $request->request->all();
            $email= $data['form']['email'];
            $password= $data['form']['password'];
            $phone= $data['phone'];
            $firstname = $passagers[0]->getFirstname();
            $lastname = $passagers[0]->getLastname();
            $usr= $userRepository->findOneBy(['email'=>$email]);

            if ($usr == null){
                $user = new User();
                $user->setIsVerified(false);
                $user->setCreatedAt(new \DateTime());
                $user->setUpdatedAt(new \DateTime());
                $user->setPlainPassword($password);
                $user->setEmail($email);
                $user->setPhone($phone);
                $user->setFirstname($firstname);
                $user->setLastname($lastname);
                $user->setPassword($userPasswordHasher->hashPassword($user,$password));
                $entityManager->persist($user);
                $booking->setUser($user);
                $entityManager->persist($booking);
                $entityManager->flush();
                $userAuthenticator->authenticateUser(
                    $user,
                    $authenticator,
                    $request
                );
                
            }else{
                $request->getSession()->set('_referer', '/payment/status/'.$booking->getNumdossier());
                //$this->saveTargetPath($request->getSession(), 'main', '/payment/status/'.$booking->getNumdossier());
                return $this->redirectToRoute('app_login');
            }
            return $this->redirectToRoute('app_payment_status', ['id' => $booking->getNumdossier()], Response::HTTP_SEE_OTHER);

        }
        if ($user){
            if ($booking->getUser()==null){
                $booking->setUser($user);
                $entityManager->persist($booking);
                $entityManager->flush();
                $this->envoiMail($booking, $mailer, $passagers);
            }
        }
        $qrcode = '/assets/qr-code/'.$booking->getQrcode();
        return $this->render('payment/status.html.twig', [
            'form'=>$form,
            //'user'=>$this->getUser(),
            'qrcode' => $qrcode,
            'passagers' => $passagers,
            'booking' => $booking,
        ]);    
    }


  //  #[Route('/reqstatus/{id}', name: 'app_payment_reqstatus', methods: ['GET', 'POST'])]
    public function requestToPayStatus($apiurl, $uuid, $acs_token, $oask, $t_env)
    {
        $request = new HTTP_Request2($apiurl.'/collection/v1_0/requesttopay/'. $uuid );
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'Authorization' => 'Bearer '.$acs_token,
            'X-Target-Environment' => $t_env, // mtncongo en prod
            'Ocp-Apim-Subscription-Key' => $oask,
        );

        $request->setHeader($headers);
        $parameters = array(
            // Request parameters
        );
        $url->setQueryVariables($parameters);
        $request->setMethod(HTTP_Request2::METHOD_GET);
        // Request body
        $request->setBody('');

        return $request;
 
    }

    public function requestToPay($apiurl, $montant, $currency, $phone, $uuid, $acs_token, $oask, $t_env)
    {
       //-------------------------------------------------------------------------
  
       $request = new HTTP_Request2($apiurl.'/collection/v1_0/requesttopay');
       $url = $request->getUrl();
       if ($t_env=='mtncongo'){
           $headers = array(
                // Request headers
                'Authorization' => 'Bearer '.$acs_token,
                'X-Callback-Url' => 'https://kerenes.com/module/mobilepayment/api_success',
                'X-Reference-Id' => $uuid,
                'X-Target-Environment' => $t_env, // mtncongo en mode prod
                'Content-Type' => 'application/json',
                'Ocp-Apim-Subscription-Key' => $oask,
            );
       }else{
           $headers = array(
                // Request headers
                'Authorization' => 'Bearer '.$acs_token,
                //'X-Callback-Url' => $callback,
                'X-Reference-Id' => $uuid,
                'X-Target-Environment' => $t_env, // mtncongo en mode prod
                'Content-Type' => 'application/json',
                'Ocp-Apim-Subscription-Key' => $oask,
           );
       }

       $request->setHeader($headers);
       $parameters = array(
           // Request parameters
       );

       $url->setQueryVariables($parameters);
       $request->setMethod(HTTP_Request2::METHOD_POST);
       $bdy = array(
           // Request body
           'amount'=> $montant,
           'currency'=> $currency,
           'externalId'=> $uuid,
           'payer'=> array(
               'partyIdType'=> 'MSISDN',
               'partyId'=> $phone,
                   ),
           'payerMessage'=> 'Facture de Kerenes Digital',
           'payeeNote'=> 'Paiement',  
       );


       // Request body
       try
       {
           $body=json_encode($bdy);
           $request->setBody($body);
           $response = $request->send();
           $code = $response->getStatus();  
           return $code;

       }
       catch (HttpException $ex)
       {
           echo 'erreur'.$ex;
       }

       return 0;
    }

    public function accessToken($apiurl, $apiuser, $apikey, $oask)
    {   
        $request = new HTTP_Request2($apiurl.'/collection/token/');
        $url = $request->getUrl();
        $token = base64_encode($apiuser.':'.$apikey);
        $headers = array(
            // Request headers
            'Authorization' => 'Basic '. $token,
            'Ocp-Apim-Subscription-Key' => $oask,
        );
        $request->setHeader($headers);
        $parameters = array(
            // Request parameters
        );
        $url->setQueryVariables($parameters);

        $request->setMethod(HTTP_Request2::METHOD_POST);
        // Request body
        $request->setBody("");
        try
        {
            $response = $request->send();
            $access_token = json_decode($response->getBody());
            $acs_token = $access_token->access_token;
            return $acs_token;
        }
        catch (HttpException $ex)
        {
            echo $ex;
        }
        return 0;
    }

    public function createFacturePdf(Booking $booking, array $passagers)
    {   
        // création du pdf
        $pdfOptions = new Options();
        $pdfOptions->set('defaultFont', 'Arial');
        $dompdf = new Dompdf($pdfOptions);

        $path = $this->getParameter('kernel.project_dir').'/public/assets/img/gallery/logoKD-146x112.png';
        $type = pathinfo($path, PATHINFO_EXTENSION);
        $data = file_get_contents($path);
        $logobase64 = 'data:image/' . $type . ';base64,' . base64_encode($data);

        if ($booking->getCompany()->getImage()===null){
            $pathlogoCompany = $this->getParameter('kernel.project_dir').'/public/assets/img/company/0.jpg';
        }else{
            $pathlogoCompany = $this->getParameter('kernel.project_dir').'/public/assets/img/company/'.$booking->getCompany()->getImage();
        }
        $type = pathinfo($pathlogoCompany, PATHINFO_EXTENSION);
        $data = file_get_contents($pathlogoCompany);
        $logocompany64 = 'data:image/' . $type . ';base64,' . base64_encode($data);

        $pathQrcode = $this->getParameter('kernel.project_dir').'/public/assets/qr-code/'.$booking->getQrcode();
        $type = pathinfo($pathQrcode, PATHINFO_EXTENSION);
        $data = file_get_contents($pathQrcode);
        $qrcode = 'data:image/' . $type . ';base64,' . base64_encode($data);

        $user=$this->getUser();
        if ($user){
            $usr = ['firstname' =>$user->getFirstname(), 'lastname'=>$user->getLastname()];
        }else{
            $usr = ['firstname' =>$passagers[0]->getFirstname(), 'lastname'=>$passagers[0]->getLastname()];

        }
        $message = $booking->getSchedule()->getRemarks();
        // Retrieve the HTML generated in our twig file
        $html = $this->renderView('payment/facture.html.twig', [
            'user'=> $usr,
            'booking'=>$booking,
            'passagers'=>$passagers,
            'logobase64' => $logobase64,
            'logocompany64' => $logocompany64,
            'message' => $message,
            'qrcode' => $qrcode,
        ]);

        // Load HTML to Dompdf
        $dompdf->loadHtml($html);
        $dompdf->setPaper('A4', 'portrait');
        $dompdf->render();

        // Store PDF Binary Data
        $output = $dompdf->output();

        //$publicDirectory=dirname( __DIR__, 2).'/public/uploads';

        $namePdf = uniqid(prefix:'', more_entropy: false).'.pdf';        
        $publicDirectory=$this->getParameter('kernel.project_dir').'/public/uploads';
        $pdfFilepath =  $publicDirectory . '/'.$namePdf;

        // Write file to the desired path
        file_put_contents($pdfFilepath, $output);
        
        return $namePdf;

    }

    public function envoiMail(Booking $booking, MailerInterface $mailer, $passagers)
    {   
        $user= $this->getUser();

        $namePdf = $booking->getFacture();
        $publicDirectory=$this->getParameter('kernel.project_dir').'/public/uploads';
        $pdfFilepath =  $publicDirectory . '/'.$namePdf;

        if ($user){
            $email = (new TemplatedEmail())
                //->from('no-reply@kerenes-digital.com')
                ->from(new Address('no-reply@kerenes-digital.com', 'Kerene\'s Travel'))
                ->to($this->getUser()->getEmail()) 
                ->subject('Paiement via Kerenes digital')
                ->htmlTemplate('email/payment.html.twig')
                ->context([
                            'user'=> $this->getUser(),
                            'booking'=>$booking,
                            'passagers' => $passagers,
                            ])
                ->attachFromPath($pdfFilepath);
            $mailer->send($email);
        }
    }

    #[IsGranted('ROLE_USER')]
    #[Route('/', name: 'app_payment_index', methods: ['GET'])]
    public function index(BookingRepository $bookingRepository, UserRepository $ur,
        PaginatorInterface $paginator,
        Request $request
    ): Response
    {
        $data = $bookingRepository->findPayment($this->getUser()->getId());

 //       dd($data);
        $bookings = $paginator->paginate(
            $data,
            $request->query->getInt('page', 1), 10
        );

        return $this->render('payment/index.html.twig', [
            'bookings' => $bookings,
        ]);
    }

    #[Route('/{id}', name: 'app_payment_show', methods: ['GET'])]
    public function show(Payment $payment): Response
    {
        return $this->render('payment/show.html.twig', [
            'payment' => $payment,
        ]);
    }
    
    #[Route('/booking/{bid}', name: 'app_payment_booking', methods: ['GET'])]
    public function booking(Request $request, PaymentRepository $paymentRepository): Response
    {
        $bid = $request->get('bid');
        $payment = $paymentRepository->findOneBy(['booking'=>$bid]);
        return $this->render('payment/show.html.twig', [
            'payment' => $payment,
        ]);
    }

}
