<?php

namespace App\Controller;

use App\Entity\Agence;
use App\Entity\Bus;
use App\Entity\Company;
use App\Entity\CompanyVille;
use App\Entity\Employee;
use App\Entity\TypeTarif;
use App\Entity\User;
use App\Entity\Ville;
use App\Form\RegistrationFormType;
use App\Repository\BookingRepository;
use App\Repository\PaymentRepository;
use App\Repository\UserRepository;
use App\Repository\VilleRepository;
use App\Security\EmailVerifier;
use App\Security\LoginAuthAuthenticator;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Http\Authentication\UserAuthenticatorInterface;
use Symfony\Contracts\Translation\TranslatorInterface;
use SymfonyCasts\Bundle\VerifyEmail\Exception\VerifyEmailExceptionInterface;

class RegistrationController extends AbstractController
{
    private EmailVerifier $emailVerifier;

    public function __construct(EmailVerifier $emailVerifier)
    {
        $this->emailVerifier = $emailVerifier;
    }
    
    #[Route('/post-register', name: 'app_post_register')]
    public function postRegister(
        Request $request,
        BookingRepository $br,
        PaymentRepository $pr,
        UserRepository $ur,
        UserPasswordHasherInterface $userPasswordHasher,
        MailerInterface $mailer,
        UserAuthenticatorInterface $userAuthenticator,
        LoginAuthAuthenticator $authenticator,
        ResetPasswordController $resetPasswordController,
        EntityManagerInterface $entityManager): Response
    {
        
        $data= $request->request->all();
        $email= $data['email'];
        $password= $data['password'];
        $bookingId = $data['bookingId'];
        $firstname = $data['firstname'];
        $lastname = $data['lastname'];

        $booking = $br->findOneBy(['id'=> $bookingId]);
        $payment = $pr->findOneBy(['booking'=> $bookingId]);

        $usr= $ur->findOneBy(['email'=>$email]);
        if ($usr == null){
            $user = new User();
            $user->setIsVerified(false);
            $user->setCreatedAt(new \DateTime());
            $user->setUpdatedAt(new \DateTime());
            $user->setPlainPassword($password);
            $user->setEmail($email);
            $user->setFirstname($firstname);
            $user->setLastname($lastname);
            $user->setPassword($userPasswordHasher->hashPassword($user,$password));
            $entityManager->persist($user);
        }else{
            $user = $usr;
            $user->setPassword($userPasswordHasher->hashPassword($user,$password));
        }
        $uauth = $userAuthenticator->authenticateUser(
            $user,
            $authenticator,
            $request
        );
        if ($uauth->getStatusCode()==302){
            $booking->setUser($user);
            $entityManager->persist($booking);
            $entityManager->persist($payment);
            $entityManager->flush();
        }
        return $this->redirectToRoute('app_payment_status', ['id' => $bookingId], Response::HTTP_SEE_OTHER);
    }


    #[Route('/register', name: 'app_register')]
    public function register(Request $request, UserPasswordHasherInterface $userPasswordHasher, UserAuthenticatorInterface $userAuthenticator, LoginAuthAuthenticator $authenticator, EntityManagerInterface $entityManager): Response
    {
        $user = new User();
        $form = $this->createForm(RegistrationFormType::class, $user);
        $user->setIsVerified(false);
        $user->setCreatedAt(new \DateTime());
        $user->setUpdatedAt(new \DateTime());
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            // encode the plain password

            $user->setPassword(
                $userPasswordHasher->hashPassword(
                    $user,
                    $form->get('password')->getData()
                )
            );
            $entityManager->persist($user);
            $entityManager->flush();

            // generate a signed url and email it to the user
            $this->emailVerifier->sendEmailConfirmation('app_verify_email', $user,
                (new TemplatedEmail())
                    ->from(new Address('no-reply@kerenes-digital.com', 'Kerene\'s Digital'))
                    ->to($user->getEmail())
                    ->subject('Please Confirm your Email')
                    ->htmlTemplate('registration/confirmation_email.html.twig')
            );
            // do anything else you need here, like send an email

            return $userAuthenticator->authenticateUser(
                $user,
                $authenticator,
                $request
            );
        }

        return $this->render('registration/register.html.twig', [
            'registrationForm' => $form->createView(),
        ]);
    }


    
    #[Route('/company_register', name: 'app_company_register')]
    public function regcompany(Request $request,
         UserPasswordHasherInterface $userPasswordHasher,
         UserAuthenticatorInterface $userAuthenticator,
         LoginAuthAuthenticator $authenticator,
         VilleRepository $villeRepository,
         MailerInterface $mailer,
         EntityManagerInterface $entityManager): Response
    {
        $villes = $villeRepository->findAll();
        $form = $this->createFormBuilder()
            ->add('company', TextType::class,[
                'label' => 'Nom de la compangie',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'Mot de passe',
                ]
            ])
            ->add('villes', ChoiceType::class,[
                'choices'=> $villes,
                'label'=>'Villes desservies',
                'choice_value' => 'name',
                'choice_label' => function (?Ville $ville): string {
                    return $ville ? $ville->getName() : '';
                },
                'choice_attr' => function (?Ville $ville): array {
                    return $ville ? ['class' => 'category_'.strtolower($ville->getName())] : [];
                },
                'expanded'=>true,
                'multiple'=>true,
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'Mot de passe',
                ]
            ])
            ->add('lastname', TextType::class,[
                'label' => 'Nom',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'Mot de passe',
                ]
            ]) 
            ->add('firstname', TextType::class,[
                'label' => 'Prénom',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'nom@domain.com',
                ]
            ])
            ->add('email', EmailType::class,[
                'label' => 'Email',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'Mot de passe',
                ]
            ])
            ->add('password', PasswordType::class,[
                'label' => 'Mot de passe',
                'required' => true,
                'attr' =>[
                    //'class'=>'btn btn-warning',
                    //'placeholder' => 'Mot de passe',
                ]
            ])
            ->add('send', SubmitType::class,[
                'label' => 'Valider',
                'attr' =>[
                    'class'=>'btn btn-primary btn-lg btn-block',
                    'placeholder' => 'Valider',
                ]
            ])
            ->getForm();

        $employee = new Employee();
        $employee->setIsVerified(false);
        $employee->setCreatedAt(new \DateTime());
        $employee->setUpdatedAt(new \DateTime());
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            // encode the plain password
            $villes = $form->get('villes')->getData();
            
            $company = new Company();
            $company->setName($form->get('company')->getData());
            $company->setAddress('');
            $company->setEmail($form->get('email')->getData());
            $company->setTelephone('');
            $company->setCreatedAt(new \DateTime());
            $company->setUpdatedAt(new \DateTime());
            $entityManager->persist($company);

            foreach ($villes as $ville) {
                $companyVille = new CompanyVille();
                $companyVille->setVille($ville);
                $companyVille->setCompany($company);
                $entityManager->persist($companyVille);

                $agence = new Agence();
                $agence->setCompanyVille($companyVille);
                $agence->setCompany($company);
                $agence->setName($ville->getName());
                $entityManager->persist($agence);

            }

            $typeTarif = new TypeTarif();
            $typeTarif->setName('Normal');
            $typeTarif->setCompany($company);
            $entityManager->persist($typeTarif);

            $employee->setFirstname($form->get('firstname')->getData());
            $employee->setLastname($form->get('lastname')->getData());
            $employee->setEmail($form->get('email')->getData());
            $employee->setRoles(["ROLE_ADMIN"]);
            $employee->setCountry($form->get('firstname')->getData());
            $employee->setPhone('');
            $employee->setCompany($company);
            $employee->setFirstname($form->get('firstname')->getData());
            $employee->setPassword(
                $userPasswordHasher->hashPassword(
                    $employee,
                    $form->get('password')->getData()
                )
            );
            $entityManager->persist($employee);

            $bus = new Bus();
            $bus->setCompany($company);
            $bus->setEmployee($employee);
            $bus->setBusPlateNumber('123');
            $bus->setCapacity(30);
            $bus->setCreatedAt(new \DateTime());
            $bus->setUpdatedAt(new \DateTime());
            $entityManager->persist($bus);

            $entityManager->flush();

            $email = (new TemplatedEmail())
            //->from('no-reply@kerenes-digital.com')
            ->from($employee->getEmail())
            ->to(new Address('contact@kerenes-digital.com', 'Kerene\'s Travel')) 
            ->subject('Inscription compagnie')
            ->htmlTemplate('company/email_register.html.twig');

            $mailer->send($email);
                        // // generate a signed url and email it to the user

            return $this->render('company/register_success.html.twig', [
            ]); 

            // // do anything else you need here, like send an email

            // return $userAuthenticator->authenticateUser(
            //     $employee,
            //     $authenticator,
            //     $request
            // );

        }

        return $this->render('company/register.html.twig', [
            'form' => $form,
        ]);
    }

    #[Route('/verify/email', name: 'app_verify_email')]
    public function verifyUserEmail(Request $request, TranslatorInterface $translator): Response
    {
 
        $this->denyAccessUnlessGranted('IS_AUTHENTICATED_FULLY');
        // validate email confirmation link, sets User::isVerified=true and persists
        try {
            $this->emailVerifier->handleEmailConfirmation($request, $this->getUser());
        } catch (VerifyEmailExceptionInterface  $exception) {
            $this->addFlash('verify_email_error', $translator->trans($exception->getReason(), [], 'VerifyEmailBundle'));
            return $this->redirectToRoute('app_register');
        }

        // @TODO Change the redirect on success and handle or remove the flash message in your templates
        $this->addFlash('success', 'Votre adresse email a été vérifiée.');

        return $this->redirectToRoute('app_home');
    }
}
