<?php

namespace App\Controller;

use App\Entity\Schedule;
use App\Form\SearchType;
use App\Repository\CompanyRepository;
use App\Repository\ScheduleRepository;
use App\Repository\TarifRepository;
use DateTime;
use Knp\Component\Pager\PaginatorInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

#[Route('/schedule')]
class ScheduleController extends AbstractController
{
    #[Route('/', name: 'app_schedule_index', methods: ['GET','POST'])]
    //#[IsGranted("ROLE_ADMIN")]
    public function index(
        ScheduleRepository $scheduleRepository,
        CompanyRepository $companyRepository,
        TarifRepository $tarifRepository,
        PaginatorInterface $paginator,
        TranslatorInterface $translator,
        Request $request): Response
    {

        $companyId=0;
        if ($request->query->get('company')){
            $companyId=$request->query->get('company');
        };
        $date = new DateTime();
        $search = new Schedule();
        $data = $scheduleRepository->findAllSort($companyId);
        $form = $this->createForm(SearchType::class, $search);
        $form->handleRequest($request);
        $paramsForm = $request->request->all();
        if (isset($paramsForm['search']['depart']) && isset($paramsForm['search']['arrivee']))
        {
             $depart = $paramsForm['search']['depart'];
             $arrivee = $paramsForm['search']['arrivee'];
             $date = $paramsForm['search']['date'];
             $data = $scheduleRepository->findSchedule($date, $depart, $arrivee, $companyId);
        }
        $schedules = $paginator->paginate(
            $data,
            $request->query->getInt('page', 1), 25
        );
        
        $companies=[];
        $data2 = $scheduleRepository->findAllSort(0);
        foreach ($data2 as $schedule){
            $company = $companyRepository->findOneBy(['id'=>$schedule->getCompany()->getId()]);
            $companies[$company->getId()] = $company;
        }
        $tarifs = $tarifRepository->findAll();
        return $this->render('schedule/index.html.twig', [
            'companies' => $companies,
            'schedules' => $schedules,
            'tarifs' => $tarifs,
            'form' => $form->createView()
        ]);
    }

    #[Route('/{id}', name: 'app_schedule_show', methods: ['GET'])]
    public function show(Schedule $schedule): Response
    {
        return $this->render('schedule/show.html.twig', [
            'schedule' => $schedule,
        ]);
    }

}
