<?php

namespace App\Entity;

use App\Repository\VilleRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity(repositoryClass: VilleRepository::class)]
class Ville
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    private ?string $name = null;

    #[ORM\Column(length: 50)]
    private ?string $abbreviation = null;

    #[ORM\OneToMany(mappedBy: 'depart', targetEntity: Schedule::class)]
    private Collection $schedules;

    #[ORM\OneToMany(mappedBy: 'depart', targetEntity: Tarif::class)]
    private Collection $tarifs;

    #[ORM\OneToMany(mappedBy: 'ville', targetEntity: CompanyVille::class)]
    private Collection $companyVilles;

    public function __construct()
    {
        $this->schedules = new ArrayCollection();
        $this->tarifs = new ArrayCollection();
        $this->companyVilles = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): static
    {
        $this->name = $name;

        return $this;
    }

    public function getAbbreviation(): ?string
    {
        return $this->abbreviation;
    }

    public function setAbbreviation(string $abbreviation): static
    {
        $this->abbreviation = $abbreviation;

        return $this;
    }


    public function __toString() {
        return $this->name;
    }

    /**
     * @return Collection<int, Schedule>
     */
    public function getSchedules(): Collection
    {
        return $this->schedules;
    }

    public function addSchedule(Schedule $schedule): static
    {
        if (!$this->schedules->contains($schedule)) {
            $this->schedules->add($schedule);
            $schedule->setDepart($this);
        }

        return $this;
    }

    public function removeSchedule(Schedule $schedule): static
    {
        if ($this->schedules->removeElement($schedule)) {
            // set the owning side to null (unless already changed)
            if ($schedule->getDepart() === $this) {
                $schedule->setDepart(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Tarif>
     */
    public function getTarifs(): Collection
    {
        return $this->tarifs;
    }

    public function addTarif(Tarif $tarif): static
    {
        if (!$this->tarifs->contains($tarif)) {
            $this->tarifs->add($tarif);
            $tarif->setDepart($this);
        }

        return $this;
    }

    public function removeTarif(Tarif $tarif): static
    {
        if ($this->tarifs->removeElement($tarif)) {
            // set the owning side to null (unless already changed)
            if ($tarif->getDepart() === $this) {
                $tarif->setDepart(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, CompanyVille>
     */
    public function getCompanyVilles(): Collection
    {
        return $this->companyVilles;
    }

    public function addCompanyVille(CompanyVille $companyVille): static
    {
        if (!$this->companyVilles->contains($companyVille)) {
            $this->companyVilles->add($companyVille);
            $companyVille->setVille($this);
        }

        return $this;
    }

    public function removeCompanyVille(CompanyVille $companyVille): static
    {
        if ($this->companyVilles->removeElement($companyVille)) {
            // set the owning side to null (unless already changed)
            if ($companyVille->getVille() === $this) {
                $companyVille->setVille(null);
            }
        }

        return $this;
    }

}
