<?php

namespace App\Repository;

use App\Entity\Booking;
use Doctrine\ORM\Query\Expr\Join;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @extends ServiceEntityRepository<Booking>
 *
 * @method Booking|null find($id, $lockMode = null, $lockVersion = null)
 * @method Booking|null findOneBy(array $criteria, array $orderBy = null)
 * @method Booking[]    findAll()
 * @method Booking[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class BookingRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Booking::class);
    }

    public function save(Booking $entity, bool $flush = false): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(Booking $entity, bool $flush = false): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

   /**
    * @return Booking[] Returns an array of Booking objects
    */

    
    public function findPayment($userId): array
    {
        $value=0;

        return $this->createQueryBuilder('b')
            ->where('b.user = :uid')
            ->andwhere('b.facture is not null')
            ->setParameter('uid', $userId)
            ->getQuery()
            ->getResult()
        ;
    }
 
//    public function findForPagination($value): array
//    {
//        return $this->createQueryBuilder('u')
//            ->andWhere('u.exampleField = :val')
//            ->setParameter('val', $value)
//            ->orderBy('u.id', 'ASC')
//            ->setMaxResults(5)
//            ->getQuery()
//            ->getResult()
//        ;
//    }


public function findManifesteByDate($date, $company): array
{
    return $this->createQueryBuilder('b')    
        ->leftJoin('App\Entity\Schedule', 's', Join::WITH, 's.id = b.schedule')
        ->leftJoin('App\Entity\User', 'u', Join::WITH, 'u.id = b.user')
        ->andWhere('s.scheduleDate = :date')
        ->andWhere('s.company = :company')
        ->andWhere('b.facture is not null')
        ->setParameter('date', $date)
        ->setParameter('company', $company)
        ->orderBy('u.lastname', 'ASC')
        ->getQuery()
        ->getResult()
    ;
}
   public function findByDate($from, $to, $companies): array
   {
       return $this->createQueryBuilder('u')
           ->andWhere('u.dateOfBooking >= :from')
           ->andWhere('u.dateOfBooking <= :to')
           ->andWhere('u.company in (:companies)')
           ->setParameter('from', $from)
           ->setParameter('to', $to)
           ->setParameter('companies', $companies)
           ->orderBy('u.dateOfBooking', 'ASC')
           ->getQuery()
           ->getResult()
       ;
   }
//    public function findOneBySomeField($value): ?Booking
//    {
//        return $this->createQueryBuilder('u')
//            ->andWhere('u.exampleField = :val')
//            ->setParameter('val', $value)
//            ->getQuery()
//            ->getOneOrNullResult()
//        ;
//    }
}
